/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*-
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

[scriptable, uuid(351887ca-56b2-4458-96fc-88baeb57b6e7)]
interface nsISidebar : nsISupports
{
  void addSearchEngine(in DOMString engineURL, in DOMString iconURL,
                       in DOMString suggestedTitle, in DOMString suggestedCategory);
};

[scriptable, uuid(5895076f-e28e-434a-9fdb-a69f94eb323f)]
interface nsISidebarExternal : nsISupports
{
  void AddSearchProvider(in DOMString aDescriptionURL);
  unsigned long IsSearchProviderInstalled(in DOMString aSearchURL);
};

%{ C++
// {577CB744-8CAF-11d3-AAEF-00805F8A4905} 
#define NS_SIDEBAR_CID \
{ 0x577cb744, 0x8caf, 0x11d3, { 0xaa, 0xef, 0x0, 0x80, 0x5f, 0x8a, 0x49, 0x5 } }

#define NS_SIDEBAR_CONTRACTID "@mozilla.org/sidebar;1"
%}
